/////////////////////////////////////////////////////////////////////
//
//  Copyright (c) 2004 Mothers Ruin Software. All rights reserved.
//
//  MODULE:  EnvFile.h
//
//  PROJECT: SSHChain
//
//  CREATED: by randy on Thu Feb 19 2004
//
//  $Id: EnvFile.h,v 1.3 2004/04/04 00:16:21 randy Exp $
//
//  SSHChain is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  SSHChain is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SSHChain; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
/////////////////////////////////////////////////////////////////////

#import <Foundation/Foundation.h>
@class SSHAgent;

/////////////////////////////////////////////////////////////////////
// class EnvFile
/////////////////////////////////////////////////////////////////////

/*!
 * @class EnvFile
 *
 * The EnvFile class manages the environment file which is intended
 * to be included by the user's shell startup file (e.g. .cshrc).
 * A single EnvFile object manages an environment file at a specific
 * path, with a specific format (csh- or sh-compatible). It can be
 * updated at any time with the information from a given SSHAgent
 * object. Upon logout (or change in the environment file prefs), it
 * handles removal of the file.
 */
@interface EnvFile : NSObject
{
   NSString* m_path;
   BOOL      m_csh;
}

/*!
 * @method initWithPath:cshFormat:
 *
 * The initWithPath:cshFormat: method is the designated initializer.
 * It specifies the path of the environment file and its format.
 * The directory portion of the given file path must already exist,
 * although the file will usually not (and will be overwritten if 
 * it does exist).
 *
 * TODO: check for existing and writable directory, or return nil
 *
 * The actual file will not be written until the EnvFile object is
 * associated with an SSHAgent, via updateForAgent:.
 *
 * @param path
 *    path to environment file to write
 * @param csh
 *    YES to use csh format, NO to use sh format
 * @result
 *    initialized EnvFile object, or nil if error
 */
- (id) initWithPath:(NSString*)path
       cshFormat:(BOOL)csh;

/*!
 * @method updateForAgent:
 *
 * Update the environment file with the given SSHAgent's socket and
 * PID information. This will cause the environment file to be
 * written the first time it is called, and rewritten on subsequent
 * invocations. If the given SSHAgent is not running (valid), the
 * environment file will be deleted (as in clear).
 *
 * @param agent
 *    the agent to be set in the environment
 */
- (void) updateForAgent:(SSHAgent*)agent;

/*!
 * @method clear
 *
 * Remove the environment file. Appropriate when the agent is being
 * shut down. Equivalent to updateForAgent: with an invalid agent.
 */
- (void) clear;

@end
