/////////////////////////////////////////////////////////////////////
//
//  Copyright (c) 2004 Mothers Ruin Software. All rights reserved.
//
//  MODULE:  EnvFile.m
//
//  PROJECT: SSHChain
//
//  CREATED: by randy on Thu Feb 19 2004
//
//  $Id: EnvFile.m,v 1.3 2007/10/20 23:38:14 randy Exp $
//
//  SSHChain is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  SSHChain is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SSHChain; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
/////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////
// include files
/////////////////////////////////////////////////////////////////////

#import "EnvFile.h"
#import "SSHAgent.h"

/////////////////////////////////////////////////////////////////////
// class EnvFile
/////////////////////////////////////////////////////////////////////

@interface EnvFile ( PrivateMethods )
- (void) writeEnvForAgent:(SSHAgent*)agent;
- (void) deleteEnv;
@end

@implementation EnvFile

- (id) initWithPath:(NSString*)path
       cshFormat:(BOOL)csh
{
   if ( self = [super init] )
   {
      m_path = [[path stringByExpandingTildeInPath] retain];
      m_csh = csh;
   }
   return( self );
}

- (void) dealloc
{
   [m_path release];
   [super dealloc];
}

- (void) updateForAgent:(SSHAgent*)agent
{
   if ( [agent isValid] )
      [self writeEnvForAgent:agent];
   else
      [self deleteEnv];
}

- (void) clear
{
   [self deleteEnv];
}

@end

@implementation EnvFile ( PrivateMethods )

- (void) writeEnvForAgent:(SSHAgent*)agent
{
   // construct format
   NSString* fmt = nil;
   if ( m_csh )
   {
      fmt = @"setenv SSH_AUTH_SOCK %@;\n"
            @"setenv SSH_AGENT_PID %d\n";
   }
   else
   {
      fmt = @"SSH_AUTH_SOCK=%@; export SSH_AUTH_SOCK;\n"
            @"SSH_AGENT_PID=%d; export SSH_AGENT_PID\n";
   }

   // get the agent information to write to the file
   NSString* authSocket = [agent authSocket];
   int agentPID = [agent agentPID];
   
   // construct as string in right format
   NSString* script = [NSString stringWithFormat:fmt,
                                authSocket,
                                agentPID];
   
   // write to the file
   [script writeToFile:m_path atomically:YES];
}

- (void) deleteEnv
{
   NSFileManager* fm = [NSFileManager defaultManager];
   BOOL dir = YES;
   if ( [fm fileExistsAtPath:m_path isDirectory:&dir] && ! dir )
   {
      [fm removeFileAtPath:m_path handler:nil];
   }
}

@end
