/////////////////////////////////////////////////////////////////////
//
//  Copyright (c) 2004 Mothers Ruin Software. All rights reserved.
//
//  MODULE:  PrefController.m
//
//  PROJECT: SSHChain
//
//  CREATED: by randy on Thu Jan 13 2004
//
//  $Id: PrefController.m,v 1.17 2007/10/02 04:50:18 randy Exp $
//
//  SSHChain is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  SSHChain is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SSHChain; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
/////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////
// include files
/////////////////////////////////////////////////////////////////////

#import "PrefController.h"
#import "LoginItemsAE.h"

/////////////////////////////////////////////////////////////////////
// class PrefController
/////////////////////////////////////////////////////////////////////

NSString* SavePreferencesNotification     = @"SavePreferencesNotification";
NSString* AgentRestartRequestNotification = @"AgentRestartRequestNotification";
NSString* SocketPathChangedNotification   = @"SocketPathChangedNotification";
NSString* AgentEnvFileChangedNotification = @"AgentEnvFileChangedNotification";

#define PREF_SOCKET_PATH_KEY     @"SocketPath"
#define PREF_ENV_PATH_KEY        @"EnvFile"
#define PREF_ENV_SHELL_KEY       @"EnvShell"
#define PREF_ENV_CSH_KEY         @"csh"
#define PREF_ENV_SH_KEY          @"sh"
#define PREF_VERSION_KEY         @"PrefVersion"

#define LW_LOGIN_ITEMS           @"AutoLaunchedApplicationDictionary"
#define LW_APP_NAME              @"loginwindow"

static PrefController* s_prefs = NULL;

@interface PrefController ( PrivateMethods )
+ (NSDictionary*) factoryDefaults;
- (int) loginItemIndex;
- (void) addToLoginItems;
- (BOOL) removeFromLoginItems;
- (void) updateUIFromDefaults;
- (void) updateDefaultsFromUI;
- (void) configureAboutBox;
@end

@implementation PrefController

+ (PrefController*) sharedPrefController
{
   if ( s_prefs == NULL )
      s_prefs = [[PrefController alloc] init];
   return( s_prefs );
}

+ (void) initialize
{
   NSUserDefaults* d = [NSUserDefaults standardUserDefaults];
   [d registerDefaults:[PrefController factoryDefaults]];
}

- (id) init
{
   if ( self = [super initWithWindowNibName:@"Preferences"] )
   {
   }
   return( self );
}

- (void) dealloc
{
   [super dealloc];
}

- (void) upgradePreferences
{
   // extract the version of the preferences and the app
   NSUserDefaults* d = [NSUserDefaults standardUserDefaults];
   int prefVer = [d integerForKey:PREF_VERSION_KEY];
   int bundleVer = [[[NSBundle mainBundle]
                      objectForInfoDictionaryKey:@"CFBundleVersion"] intValue];

   // if preference version is same as app version, nothing to do
   if ( prefVer != 0 && prefVer == bundleVer )
      return;

   // if versions don't match (or preference version does not exist,
   // meaning the prefs predate this scheme), ugrade the prefs
   NSLog( @"Upgrading preferences from version %d to version %d",
          prefVer, bundleVer );

   // between v5 and v6, we had to move the Helper app from Resources
   // into SharedSupport, because of code signing on Leopard: fix up
   // the path in Login Items if necessary
   if ( prefVer < 6 )
   {
      // the remove search finds any SSHChainHelper.app item, regardless
      // of path, so if we are able to remove something, we were in the
      // Login Items before, so put it back with the current path
      if ( [self removeFromLoginItems] )
         [self addToLoginItems];
   }

   // update the prefs version so that we only do this once
   [d setInteger:bundleVer forKey:PREF_VERSION_KEY];
}

- (NSString*) explicitSocketPath
{
   NSUserDefaults* d = [NSUserDefaults standardUserDefaults];
   NSString* p = [d objectForKey:PREF_SOCKET_PATH_KEY];
   return( [p isEqualToString:@""] ? nil : p );
}

- (void) setExplicitSocketPath:(NSString*)path
{
   NSUserDefaults* d = [NSUserDefaults standardUserDefaults];
   if ( path == nil )
      path = @"";
   NSString* old = [[d objectForKey:PREF_SOCKET_PATH_KEY] retain];
   [d setObject:path forKey:PREF_SOCKET_PATH_KEY];

   if ( old != nil && ! [path isEqualToString:old] )
      [[NSNotificationCenter defaultCenter]
         postNotificationName:SocketPathChangedNotification
         object:self];
   [old release];
}

- (NSString*) agentEnvironmentFile
{
   NSUserDefaults* d = [NSUserDefaults standardUserDefaults];
   NSString* p = [d objectForKey:PREF_ENV_PATH_KEY];
   return( [p isEqualToString:@""] ? nil : p );
}

- (void) setAgentEnvironmentFile:(NSString*)path
{
   NSUserDefaults* d = [NSUserDefaults standardUserDefaults];
   if ( path == nil )
      path = @"";
   NSString* old = [[d objectForKey:PREF_ENV_PATH_KEY] retain];
   [d setObject:path forKey:PREF_ENV_PATH_KEY];

   if ( old != nil && ! [path isEqualToString:old] )
      [[NSNotificationCenter defaultCenter]
         postNotificationName:AgentEnvFileChangedNotification
         object:self];
   [old release];
}

- (PrefControllerAgentShell) agentEnvironmentShell
{
   NSUserDefaults* d = [NSUserDefaults standardUserDefaults];
   NSString* p = [d objectForKey:PREF_ENV_SHELL_KEY];
   if ( [p isEqualToString:PREF_ENV_SH_KEY] )
      return( AGENT_SHELL_SH );
   return( AGENT_SHELL_CSH );
}

- (void) setAgentEnvironmentShell:(PrefControllerAgentShell)shell
{
   NSUserDefaults* d = [NSUserDefaults standardUserDefaults];
   NSString* ss = ( shell == AGENT_SHELL_CSH
                    ? PREF_ENV_CSH_KEY : PREF_ENV_SH_KEY );
   NSString* old = [[d objectForKey:PREF_ENV_SHELL_KEY] retain];
   [d setObject:ss forKey:PREF_ENV_SHELL_KEY];

   if ( old != nil && ! [ss isEqualToString:old] )
      [[NSNotificationCenter defaultCenter]
         postNotificationName:AgentEnvFileChangedNotification
         object:self];
   [old release];
}

- (void) windowDidLoad
{
   // this seems to be the earliest we can successfully upgrade the
   // preferences, because the LoginItemsAE methods don't work upon
   // applicationDidLaunch: for some inexplicable reason; of course,
   // we don't get here unless someone double-clicks the wrapper
   // bundle, which isn't a problem for the Login Items upgrade, but
   // could be problematic if we ever do any other preference upgrades!
   [self upgradePreferences];

   [self updateUIFromDefaults];
   [self configureAboutBox];
}

- (IBAction) choosePath:(id)sender
{
   NSSavePanel* sp = [NSSavePanel savePanel];
   [sp setRequiredFileType:nil]; // any file type
   [sp setTitle:@"Choose Path"]; // button name: doesn't take
   [sp beginSheetForDirectory:nil
       file:@""
	   modalForWindow:[self window]
	   modalDelegate:self
	   didEndSelector:@selector( choosePathDidEnd:returnCode:contextInfo: )
	   contextInfo:sender];   
}

- (void) choosePathDidEnd:(NSSavePanel*)panel
		 returnCode:(int)returnCode
		 contextInfo:(void*)contextInfo
{
   id sender = contextInfo;
   if ( returnCode == NSOKButton )
   {
      NSString* path = [panel filename];
	  if ( sender == m_sockPathButton )
	     [m_sockPathTextField setStringValue:path];
	  else // m_envPathButton
	     [m_envPathTextField setStringValue:path];
   }
}

- (IBAction) changeSockPath:(id)sender
{
   BOOL explicit = ( [[m_sockPathRadio selectedCell] tag] == 1 );
   [m_sockPathTextField setEnabled:explicit];
   [m_sockPathButton setEnabled:explicit];
}

- (IBAction) changeEnvFile:(id)sender
{
   BOOL en = ( [m_createEnvCheckbox state] == NSOnState );
   [m_envPathTextField setEnabled:en];
   [m_envFormatRadio setEnabled:en];
   [m_envPathButton setEnabled:en];
}

- (IBAction) quitProgram:(id)sender
{
   NSBeginAlertSheet( @"Restart SSH Agent",
                      @"Cancel",
                      @"Quit SSHChain",
                      @"Restart",
                      [self window],
                      self,
      @selector( quitConfirmSheetDidEnd:returnCode:contextInfo: ),
                      NULL,
                      NULL,
                      @"Restart ssh-agent and reload SSH identities "
                      @"from the Keychain?" );
                      
}

- (void) quitConfirmSheetDidEnd:(NSWindow*)sheet
         returnCode:(int)returnCode
         contextInfo:(void*)contextInfo
{
   switch ( returnCode )
   {
   case NSAlertAlternateReturn:
      [NSApp terminate:self];
	  break;
   case NSAlertOtherReturn:
	  [[NSNotificationCenter defaultCenter]
         postNotificationName:AgentRestartRequestNotification
         object:self];
      break;
   }
}

- (IBAction) showAboutPanel:(id)sender
{
   [NSApp beginSheet:m_aboutWindow
          modalForWindow:[self window]
          modalDelegate:nil
          didEndSelector:NULL
          contextInfo:NULL];
}

- (IBAction) dismissAboutPanel:(id)sender
{
   [NSApp endSheet:m_aboutWindow];
   [m_aboutWindow orderOut:self];
}

- (IBAction) savePrefs:(id)sender
{
   // make sure to order out before posting any notifications,
   // since notifications may result in ordering back in
   [[self window] orderOut:self];
   [self updateDefaultsFromUI];

   // notify that preferences have been saved (whether or not changed)
   [[NSNotificationCenter defaultCenter]
      postNotificationName:SavePreferencesNotification object:self];
}

- (IBAction) discardPrefs:(id)sender
{
   [[self window] orderOut:self];
   // one way to make sure it comes back matching
   // the defaults; is there is smarter way?
   [self updateUIFromDefaults];
}

- (IBAction) showWebPage:(id)sender
{
   NSURL* url = [NSURL URLWithString:[sender title]];
   [[NSWorkspace sharedWorkspace] openURL:url];
}

@end

@implementation PrefController ( PrivateMethods )

+ (NSDictionary*) factoryDefaults
{
   NSMutableDictionary* d = [NSMutableDictionary dictionary];

   // default for socket path is ~/.ssh-agent-socket
   NSString* sockPath = NSHomeDirectory();
   sockPath = [sockPath stringByAppendingPathComponent:@".ssh-agent-socket"];
   sockPath = [sockPath stringByExpandingTildeInPath];
   [d setObject:sockPath forKey:PREF_SOCKET_PATH_KEY];

   // default for env file is empty, meaning don't make it
   [d setObject:@"" forKey:PREF_ENV_PATH_KEY];
   [d setObject:PREF_ENV_CSH_KEY forKey:PREF_ENV_SHELL_KEY];

   return( d );
}

- (int) loginItemIndex
{
   CFArrayRef itemsPtr = NULL;
   OSStatus err = LIAECopyLoginItems( &itemsPtr );
   if ( err != noErr || itemsPtr == NULL )
      return( NSNotFound );

   int itemIndex = NSNotFound;
   NSArray* loginItems = (NSArray*)itemsPtr;
   int i;
   for ( i = 0 ; i < [loginItems count] ; ++i )
   {
      NSDictionary* item = [loginItems objectAtIndex:i];
      NSURL* itemURL = (NSURL*)[item objectForKey:(NSString*)kLIAEURL];
      if ( [[[itemURL path] lastPathComponent]
              isEqualToString:@"SSHChainHelper.app"] )
      {
         itemIndex = i;
         break;
      }
   }
   CFRelease( itemsPtr );

   return( itemIndex );
}

- (void) addToLoginItems
{
   if ( [self loginItemIndex] != NSNotFound )
      return;
   NSBundle* bundle = [NSBundle mainBundle];
   NSURL* url = [NSURL fileURLWithPath:[bundle bundlePath]];
   LIAEAddURLAtEnd( (CFURLRef)url, NO );
}

- (BOOL) removeFromLoginItems
{
   int index = [self loginItemIndex];
   if ( index != NSNotFound )
      LIAERemove( index );
   return( index != NSNotFound ); // removed?
}

- (void) updateUIFromDefaults
{
   // login item section
   BOOL login = ( [self loginItemIndex] != NSNotFound );
   [m_loginItemCheckbox setState:( login ? NSOnState : NSOffState )];
   
   // socket path section
   NSString* sockPath = [self explicitSocketPath];
   BOOL enable = ( sockPath != nil );
   [m_sockPathRadio selectCellWithTag:( enable ? 1 : 0 )];
   if ( sockPath == nil )
      sockPath = @""; // for display
   [m_sockPathTextField setStringValue:sockPath];
   [m_sockPathTextField setEnabled:enable];
   [m_sockPathButton setEnabled:enable];

   // create environment section
   NSString* envPath = [self agentEnvironmentFile];
   PrefControllerAgentShell shell = [self agentEnvironmentShell];
   enable = ( envPath != nil );
   [m_createEnvCheckbox
      setState:( envPath != nil ? NSOnState : NSOffState )];
   if ( envPath == nil )
      envPath = @""; // for display
   [m_envPathTextField setStringValue:envPath];
   [m_envFormatRadio selectCellWithTag:shell];
   [m_envPathTextField setEnabled:enable];
   [m_envFormatRadio setEnabled:enable];
   [m_envPathButton setEnabled:enable];
}

- (void) updateDefaultsFromUI
{
   // login items
   if ( [m_loginItemCheckbox state] == NSOnState )
      [self addToLoginItems];
   else
      [self removeFromLoginItems];
   
   // socket path
   if ( [[m_sockPathRadio selectedCell] tag] == 0 )
      [self setExplicitSocketPath:nil];
   else
      [self setExplicitSocketPath:
               [m_sockPathTextField stringValue]];
   
   // environment file
   if ( [m_createEnvCheckbox state] == NSOnState )
   {
      [self setAgentEnvironmentFile:
               [m_envPathTextField stringValue]];
      [self setAgentEnvironmentShell:
               [[m_envFormatRadio selectedCell] tag]];
   }
   else
      [self setAgentEnvironmentFile:nil];
}

- (void) configureAboutBox
{
   // get the changing info from Info.plist, much like NSApplication
   // does for the standard about panel (which it does not, however,
   // make available directly for use as a sheet)
   NSBundle* bundle = [NSBundle mainBundle];
   NSString* shortVer =
      [bundle objectForInfoDictionaryKey:@"CFBundleShortVersionString"];
   NSString* bundleVer =
      [bundle objectForInfoDictionaryKey:@"CFBundleVersion"];
   NSString* copyright =
      [bundle objectForInfoDictionaryKey:@"NSHumanReadableCopyright"];
   
   // set the parts of the about box accordingly
   NSString* verStr = [NSString stringWithFormat:@"Version %@",
                                ( shortVer != nil ? shortVer : @"Unknown" )];
   if ( bundleVer != nil )
      verStr = [verStr stringByAppendingFormat:@" (v%@)", 
                       bundleVer];
   NSString* copyStr = ( copyright != nil ? copyright : @"Unknown Copyright" );
   [m_aboutVersionTextField setStringValue:verStr];
   [m_aboutCopyTextField setStringValue:copyStr];

   // make the URL button look like a link
   NSString* url = [m_aboutURLButton title];
   NSMutableParagraphStyle* style = [[NSMutableParagraphStyle alloc] init];
   [style setAlignment:NSCenterTextAlignment];
   NSDictionary* attrs = [NSDictionary dictionaryWithObjectsAndKeys:
      [NSFont systemFontOfSize:[NSFont smallSystemFontSize]],
      NSFontAttributeName,
      [NSColor blueColor],
      NSForegroundColorAttributeName,
      [NSNumber numberWithBool:YES],
      NSUnderlineStyleAttributeName,
      style,
      NSParagraphStyleAttributeName,
      nil];
   NSAttributedString* aStr = [[NSAttributedString alloc] initWithString:url
                                                          attributes:attrs];
   [m_aboutURLButton setAttributedTitle:aStr];
   [aStr release];
   [style release];

   // 10.4 gives it a focus ring unless we explicitly turn it off
   if ( [m_aboutURLButton respondsToSelector:@selector( setFocusRingType: )] )
      [m_aboutURLButton setFocusRingType:NSFocusRingTypeNone];
}

@end

