/////////////////////////////////////////////////////////////////////
//
//  Copyright (c) 2004 Mothers Ruin Software. All rights reserved.
//
//  MODULE:  SSHChainUI.m
//
//  PROJECT: SSHChain
//
//  CREATED: by randy on Wed Jan 21 2004
//
//  $Id: SSHChainUI.m,v 1.4 2007/09/22 03:28:25 randy Exp $
//
//  SSHChain is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  SSHChain is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with SSHChain; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
/////////////////////////////////////////////////////////////////////

#import <Foundation/Foundation.h>

int tellHelperTo( NSString* command );

/////////////////////////////////////////////////////////////////////
// main()
/////////////////////////////////////////////////////////////////////

int main( int argc, const char *argv[] )
{
   NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
   
   // with the helper buried inside the UI bundle, nothing seems
   // to find its script suite dictionary, and so our custom
   // command is just rejected; if we send a raw event (using the
   // event codes defined by said suite), it bypasses the check
   // and works correctly; only ASCII is properly allowed in a 
   // string literal, so we have to use Unicode for the double
   // angle quotation marks around the raw event code
   NSString* cmd = [NSString stringWithFormat:@"%Cevent MRscMRcc%C",
                             0x00AB, 0x00BB];
   tellHelperTo( cmd ); // configure sans dictionary

   [pool release];

   return( 0 );
}

/////////////////////////////////////////////////////////////////////
// tellHelperTo()
/////////////////////////////////////////////////////////////////////

int tellHelperTo( NSString* command )
{
   // determine full path to our helper; through 10.4, we put this into
   // Resources, which seemed to be the prevailing wisdom, but in 10.5,
   // code signing requires us to keep it out of Resources and sign it
   // individuually, so we use the SharedSupport directory, which is 
   // apparently for just this sort of thing (and has been there all along)
   NSBundle* bundle = [NSBundle mainBundle];
   NSString* path = [[bundle sharedSupportPath]
                       stringByAppendingPathComponent:@"SSHChainHelper.app"];
   if ( path == nil )
   {
      NSLog( @"Could not find helper in bundle!" );
      return( 1 );
   }
   
   // construct the proper script; we need to coerce the standard
   // Unix path to an AppleScript (old MacOS) style, and then
   // coerce that to a string before it can name an application
   NSString* fmt = @"set appName to \"%@\" as POSIX file\n"
                   @"tell application ( appName as string ) to %@";
   NSString* src = [NSString stringWithFormat:fmt,
                             path,
                             command];
   
   // build script object, compile and run!
   NSAppleScript* script = [[NSAppleScript alloc] initWithSource:src];
   if ( script == nil )
   {
      NSLog( @"Failed to build NSAppleScript object" );
      return( 1 );
   }
   [script autorelease];
   NSDictionary* errorInfo = nil;
   if ( [script compileAndReturnError:&errorInfo] == nil 
        || [script executeAndReturnError:&errorInfo] == nil )
   {
      NSLog( @"Error compiling script:\n%@\nerrorInfo is %@",
             src, errorInfo );
      return( 1 );
   }

   return( 0 );   
}
